#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# Name:           GFA_Report
# Usage:          Calculate total floor in simple rectangular "buildings" group.
# Date:           Jul 2011
#------------------------------------------------------------------------------------------------

require 'sketchup'

class GFA_Report

	def initialize
		@best = nil
		@mod=Sketchup.active_model
		@ent=@mod.entities
		@sel=@mod.selection
		@flr2flr=3.m if !@flr2flr
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		$gfa_dir = "c:/" if !$gfa_dir
		tfix=Time.now.to_i.to_s[3..-1]
		@gfa_file = "gfa_rpt_" + tfix + ".txt" #if !@gfa_file
		@gfa_rpt=UI.savepanel("GFA Report File:",$gfa_dir,@gfa_file)
		if !@gfa_rpt then return end
		$gfa_dir=File.dirname(@gfa_rpt)
		@gfa_out=File.open(@gfa_rpt,"w")
		@gfa_out.print("GFA Report ",Time.now,"\n\n")
		@gfa_out.close
		unit=Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
		case unit
			when 0:@ufac = 1.0;@unit="inches"
			when 1:@ufac = 12.0;@unit="feet"
			when 2:@ufac=0.03937007874;@unit="mm"
			when 3:@ufac=0.3937007874;@unit="cm"
			when 4:@ufac=39.37007874;@unit="m"
		end
		self.reset
	end

	def reset
		@pts = []
		@ip1.clear
		@sel.clear
		@drawn = false
		Sketchup::set_status_text("Select Building or Right Click to Exit", SB_PROMPT)
		Sketchup::set_status_text("", SB_VCB_LABEL)
		Sketchup::set_status_text("", SB_VCB_VALUE)
	end

	def set_current_point(x, y, view)
		if( !@ip.pick(view, x, y, @ip1) )
			return false
		end
		@ip1.copy! @ip
		need_draw = true
		view.tooltip = @ip.tooltip     
		view.invalidate if need_draw
	end

	def onMouseMove(flags, x, y, view)
		self.set_current_point(x, y, view)
		ph = view.pick_helper
		ph.do_pick(x, y)				
		@best = ph.best_picked if ph.best_picked != @best				
		Sketchup::set_status_text("Select Building or Right Click to Exit", SB_PROMPT)
		view.invalidate
	end

	def onLButtonDown(flags, x, y, view)
		if (@best.is_a?(Sketchup::Group)) then
			@sel.add @best; @bldg_name=@best.name
			ans=UI.inputbox(["Building Name:","Flr 2 Flr Dist:"],[@bldg_name, @flr2flr],"GFA Report")
			if !ans then onCancel end
			@bldg_name=ans[0].to_s; @best.name=@bldg_name
			@flr2flr=ans[1].to_l
			@gfa_out=File.open(@gfa_rpt,"a")
			@gfa_out.print("GFA Data for ",@bldg_name,"\n")
			for e in @best.entities;
				if e.is_a? Sketchup::Face;
					if e.normal.z==1; # find horizontal face
						area=e.area / @ufac**2;
						break;
					end;
				end;
			end
			height=@best.bounds.max.z - @best.bounds.min.z
			floors = (height/@flr2flr).floor; height /= @ufac; f2f = @flr2flr / @ufac; ta = area * floors
			UI.messagebox "Area:     #{area}\nHeight: #{height}\nFloor to Floor: #{f2f}\nFloors:  #{floors}\nGFA:     #{ta}"
			@gfa_out.printf("Area: %.1f%s\n",area,@unit)
			@gfa_out.printf("Height: %.1f%s\n",height,@unit)
			@gfa_out.printf("Floor to Floor: %.1f%s\n",f2f,@unit)
			@gfa_out.printf("Floors: %d\n",floors)
			@gfa_out.printf("GFA: %.1f%s\n\n",ta,@unit)
			@gfa_out.close
			self.reset
		end
		view.lock_inference
	end
	
	def onRButtonDown(flags,x,y,view)
		view.invalidate if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end
		
	def onCancel(flag, view)
		view.invalidate if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		
		# Show the current input point
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end

	end
	
end # of class GFA_Report
#----------------------------------------------------------------------------
def frametool
    Sketchup.active_model.select_tool GFA_Report.new
end

if( not file_loaded?("GFA_Report.rb") )
    UI.menu("Plugins").add_item("GFA_Report") { Sketchup.active_model.select_tool GFA_Report.new }
end
#-----------------------------------------------------------------------------
file_loaded("GFA_Report.rb") # load"GFA_Report.rb"

